library dummy;

uses
  SysUtils,
  Classes,
  tasmovies;

type
  header = record
    VBM      : array[0..3] of char;
    ver      : longword;
    uid      : longint;
    frames   : longword;
    rerecs   : longword;
    savef    : byte;
    controlf : byte;
    systemf  : byte;
    emuf     : byte;
    savetype : longword;
    flashsize: longword;
    emutype  : longword;
    gamename : array[0..11] of char;
    null     : byte;
    romcrc   : byte;
    CRC16    : word;
    gamecode : longword;
    savestate: longword;
    start    : longword;
    author   : array[0..63] of char;
    desc     : array[0..127] of char;
  end;

{$R *.res}

const
  chars='RL123456ABsS><^v';

Procedure GetInfo (ext, desc : pchar);
begin
  strcopy(ext,'vbm');
  strcopy(desc,'VisualBoy Advance movie');
end;

Procedure LoadMovie (var data: TMovie; files:integer; fn:array of pchar; res:pchar);
var
  F : file of char;
  fnn :string;
  s,nam : string;
  h : header;
  n : integer;
  cdata : array of word;
begin
data.error := 1;
if files <> 1 then exit;
//cleanup
getmem(data.filename, strlen(fn[0])+1);
strcopy(data.filename,fn[0]);
data.headers := 0;
data.frames := 0;
data.datastart := 0;
data.controllers := 0;
data.fragments := 0;
setlength(data.header,0);
setlength(data.controller,0);
setlength(data.fragment,0);
//let's go
data.controllers := 1;
fnn := fn[0];
assign(f,fnn); reset(f);
if filesize(f) < 256 then
begin
  strcopy(res,'Too small file');
  close(f); exit;
end;
blockread(f,h,256);
if h.VBM <> 'VBM'#$1a then
begin
  strcopy(res,'Wrong header');
  close(f); exit;
end;
data.headers := 21;
setlength(data.header,data.headers);
//    VBM      : array[0..3] of char;
n := 0;
getmem(data.header[0].name,7);
strcopy(data.header[0].name, 'header');
getmem(data.header[0].sdata,5);
strcopy(data.header[0].sdata, pchar(string(h.VBM)+#0));
data.header[0].tp := 3;
data.header[0].l := 4;
data.header[0].Idata := 0;
//    ver      : longword;
inc(n); nam := 'Version';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.ver);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 0;
data.header[n].l := 4;
data.header[n].Idata := h.ver;
//    uid      : longint;
inc(n); nam := 'UID';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.uid);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.uid;
//    frames   : longword;
inc(n); nam := 'frames';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.frames);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.frames;
//    rerecs   : longword;
inc(n); nam := 'Rerecordings';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.rerecs);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.rerecs;
//    savef    : byte;
inc(n); nam := 'Save flags';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.savef);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 2;
data.header[n].l := 1;
data.header[n].Idata := h.savef;
//    controlf : byte;
inc(n); nam := 'Controller flags';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.controlf);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 2;
data.header[n].l := 1;
data.header[n].Idata := h.controlf;
//    systemf  : byte;
inc(n); nam := 'System flags';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.systemf);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 2;
data.header[n].l := 1;
data.header[n].Idata := h.systemf;
//    emuf     : byte;
inc(n); nam := 'Emulator flags';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.emuf);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 2;
data.header[n].l := 1;
data.header[n].Idata := h.emuf;
//    savetype : longword;
inc(n); nam := 'Save type';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.savetype);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.savetype;
//    flashsize: longword;
inc(n); nam := 'Flash size';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.Flashsize);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.flashsize;
//    emutype  : longword;
inc(n); nam := 'Emulator type';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.emutype);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.emutype;
//    gamename : array[0..11] of char;
inc(n); nam := 'Game Name';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := h.gamename;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 3;
data.header[n].l := 4;
//    null     : byte;
inc(n); nam := 'Null';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.null);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 1;
data.header[n].Idata := h.null;
//    romcrc   : byte;
inc(n); nam := 'Rom CRC';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.romcrc);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 1;
data.header[n].Idata := h.romcrc;
//    CRC16    : word;
inc(n); nam := 'CRC16';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.crc16);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 2;
data.header[n].Idata := h.crc16;
//    gamecode : longword;
inc(n); nam := 'Game code';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.gamecode);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.gamecode;
//    savestate: longword;
inc(n); nam := 'Save state start';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.savestate);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.savestate;
//    start    : longword;
inc(n); nam := 'Data start';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := inttostr(h.start);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.start;
//    author   : array[0..63] of char;
inc(n); nam := 'Movie Author';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := h.author;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 3;
data.header[n].l := 4;
//    desc     : array[0..127] of char;}
inc(n); nam := 'Movie description';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam));
s := h.desc;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s));
data.header[n].tp := 3;
data.header[n].l := 4;
// header is ready...
data.frames := h.frames;
data.datastart := h.start;
data.fragments := 1;
setlength(data.fragment,1);
data.fragment[0].start := 256;
data.fragment[0].size := data.datastart+1-256;
getmem(data.fragment[0].name,12);
strcopy(data.fragment[0].name,'vbasavedata');
getmem(data.fragment[0].data,data.fragment[0].size);
blockread(f,data.fragment[0].data[0],data.fragment[0].size);
// on for movie
getmem(data.controller,sizeof(data.controller[0]));
getmem(data.controller[0].data,(data.frames+1)*4);
setlength(cdata,data.frames+1);
blockread(f,cdata[0],data.frames+1);
for n := 0 to data.frames do data.controller[0].data[n] := cdata[n];
//freemem(cdata);
close(f);
data.error := 0;
end;

Procedure SaveMovie (var data: TMovie; con:integer; fn: pchar; res:pchar);
begin

end;

Procedure GetCodes (s : PChar);
begin
  strcopy(s,chars);
end;

exports GetInfo, LoadMovie, GetCodes, SaveMovie;

end.
