library m64;

uses
  Sharemem,
  SysUtils,
  Classes,
  tasmovies;

type
  header = record
    m64      : array[0..3] of char;
    ver      : longword;
    uid      : longint;
    frames   : longword;
    rerecs   : longword;
    fps      : byte;
    contrnum : byte;
    res1     : word;
    inputs   : longword;
    savetype : word;
    res2     : word;
    controlf : longword;
    res3     : array[0..159] of char;
    romname  : array[0..31] of char;
    crc32    : longword;
    country  : word;
    res4     : array[0..55] of char;
    videoplg : array[0..63] of char;
    soundplg : array[0..63] of char;
    inputplg : array[0..63] of char;
    rspplg   : array[0..63] of char;
    author   : array[0..221] of char;
    desc     : array[0..255] of char;
  end;

{$R *.res}

const
  chars='><v^SZBArlduRL12xxxxxxxxyyyyyyyy';
//  chars='xxxxyyyyRL123456';

Procedure GetInfo (ext, desc : pchar);
begin
  strcopy(ext,'m64');
  strcopy(desc,'Mupen64 movie (beta)');
end;

Procedure LoadMovie (var data: TMovie; const fn: pchar; res:pchar);
var
  F : file of char;
  fnn :string;
  s,nam : string;
  h : header;
  n : integer;
  cdata : array of longword;
begin
data.error := 1;
getmem(data.filename, strlen(fn)+1);
strcopy(data.filename,fn);
data.headers := 0;
data.frames := 0;
data.datastart := 0;
data.controllers := 0;
data.fragments := 0;
setlength(data.header,0);
setlength(data.controller,0);
setlength(data.fragment,0);
//let's go
data.controllers := 1;
fnn := fn;
assign(f,fnn); reset(f);
if filesize(f) < sizeof(h) then
begin
  strcopy(res,'Too small file');
  close(f); exit;
end;
blockread(f,h,sizeof(h));
if h.m64 <> 'M64'#$1a then
begin
  strcopy(res,pchar('Wrong header'));
  close(f); exit;
end;
data.headers := 24;
setlength(data.header,data.headers);
//    VBM      : array[0..3] of char;
n := 0;
getmem(data.header[0].name,7);
strcopy(data.header[0].name, 'header');
getmem(data.header[0].sdata,5);
strcopy(data.header[0].sdata, pchar(string(h.m64)+#0));
data.header[0].tp := 3;
data.header[0].l := 4;
data.header[0].Idata := 0;
//    ver      : longword;
inc(n); nam := 'Version';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.ver);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 0;
data.header[n].l := 4;
data.header[n].Idata := h.ver;
//    uid      : longint;
inc(n); nam := 'UID';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.uid);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.uid;
//    frames   : longword;
inc(n); nam := 'Frames';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.frames);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.frames;
//    rerecs   : longword;
inc(n); nam := 'Rerecordings';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.rerecs);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.rerecs;
//    fps      : byte;
inc(n); nam := 'FPS';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.fps);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 1;
data.header[n].Idata := h.contrnum;
//    contrnum : byte;
inc(n); nam := 'Controllers';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.rerecs);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 1;
data.header[n].Idata := h.contrnum;
//    res1     : word;
inc(n); nam := 'Reserved 1';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.res1);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 2;
data.header[n].Idata := h.res1;
//    inputs   : longword;
inc(n); nam := 'Input samples';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.inputs);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.inputs;
//    savetype : word;
inc(n); nam := 'Save type';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.savetype);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 2;
data.header[n].Idata := h.savetype;
//    res2     : word;
inc(n); nam := 'Reserved 2';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.res2);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 2;
data.header[n].Idata := h.res2;
//    controlf : longword;
inc(n); nam := 'Controller Flags';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.controlf);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.controlf;
//    res3     : array[0..159] of byte;
inc(n); nam := 'Reserved 3';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.res3;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    romname  : array[0..31] of char;
inc(n); nam := 'Rom name';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.romname;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    crc32    : longword;
inc(n); nam := 'CRC32';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.crc32);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 4;
data.header[n].Idata := h.crc32;
//    country  : word;
inc(n); nam := 'Country code';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := inttostr(h.country);
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 1;
data.header[n].l := 2;
data.header[n].Idata := h.country;
//    res4     : array[0..55] of byte;
inc(n); nam := 'Reserved 4';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.res4;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    videoplg : array[0..63] of char;
inc(n); nam := 'Video Plugin';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.videoplg;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    soundplg : array[0..63] of char;
inc(n); nam := 'Sound Plugin';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.soundplg;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    inputplg : array[0..63] of char;
inc(n); nam := 'Input Plugin';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.inputplg;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    rspplg   : array[0..63] of char;
inc(n); nam := 'RSP Plugin';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.rspplg;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    author   : array[0..221] of char;
inc(n); nam := 'Author';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.author;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    desc     : array[0..255] of char;
inc(n); nam := 'Description';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := h.desc;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//    gamecode     : array[0..255] of char;
inc(n); nam := 'Controller codes';
getmem(data.header[n].name,length(nam)+1);
strcopy(data.header[n].name,pchar(nam+#0));
s := chars;
getmem(data.header[n].sdata,length(s)+1);
strcopy(data.header[n].sdata, pchar(s+#0));
data.header[n].tp := 3;
data.header[n].l := length(s);
//on for the movie
if h.ver >= 3 then
  h.inputs := (filesize(f) - $400) div 4
else
  h.inputs := (filesize(f) - $200) div 4;
data.frames := h.inputs-1;
if h.ver >= 3 then seek(f,$400) else seek(f,$200);
setlength(data.controller,1);
setlength(data.controller[0].data,(data.frames+1));
setlength(cdata,data.frames+1);
blockread(f,cdata[0],(data.frames+1)*4);
for n := 0 to data.frames do data.controller[0].data[n] := cdata[n];
//freemem(cdata);
close(f);
data.error := 0;
end;

Procedure SaveMovie (const data: TMovie; const fn: pchar; res:pchar);
var
  f : file of byte;
  fnn : string;
  cdata : array of longword;
  a : integer;
  h : header;
begin
  fillchar(h,sizeof(h),0);
  fnn := fn;
  assign(f,fnn); rewrite(f);
  //header
  h.m64 := 'M64'#$1a;

  a := getheader(data,'Version');
  if a >= 0 then
    h.ver := data.header[a].Idata
  else
    h.ver := 1;

//    uid      : longint;
  a := getheader(data,'UID');
  if a >= 0 then
    h.UID := data.header[a].Idata
  else
    h.UID := 0;

//    frames   : longword;
  h.frames := data.frames;

//    rerecs   : longword;
  a := getheader(data,'Rerecordings');
  if a >= 0 then
    h.rerecs := data.header[a].Idata
  else
    h.rerecs := 0;

//    fps      : byte;
  a := getheader(data,'FPS');
  if a >= 0 then
    h.fps := data.header[a].Idata
  else
    h.fps := 0;

//    contrnum : byte;
  a := getheader(data,'Controllers');
  if a >= 0 then
    h.contrnum := data.header[a].Idata
  else
    h.contrnum := 1;

//    res1     : word;
  a := getheader(data,'Reserved 1');
  if a >= 0 then
    h.res1 := data.header[a].Idata
  else
    h.res1 := 0;

//    inputs   : longword;
    h.inputs := data.frames+1;

//    savetype : word;
  a := getheader(data,'Save type');
  if a >= 0 then
    h.savetype := data.header[a].Idata
  else
    h.savetype := 2;

//    res2     : word;
  a := getheader(data,'Reserved 2');
  if a >= 0 then
    h.res2 := data.header[a].Idata
  else
    h.res2 := 0;

//    controlf : longword;
  a := getheader(data,'Controller Flags');
  if a >= 0 then
    h.controlf := data.header[a].Idata
  else
    h.controlf := 0;

//    res3     : array[0..159] of char;
  a := getheader(data,'Reserved 3');
  if a >= 0 then
    strcopy(@h.res3,data.header[a].Sdata)
  else
    h.res3 := '';

//    romname  : array[0..31] of char;
  a := getheader(data,'Rom name');
  if a >= 0 then
    strcopy(@h.romname,data.header[a].Sdata)
  else
    h.romname := '';

//    crc32    : longword;
  a := getheader(data,'CRC32');
  if a >= 0 then
    h.crc32 := data.header[a].Idata
  else
    h.crc32 := 0;
//    country  : word;
  a := getheader(data,'Country code');
  if a >= 0 then
    h.country := data.header[a].Idata
  else
    h.country := 0;

//    res4     : array[0..55] of char;
  a := getheader(data,'Reserved 4');
  if a >= 0 then
    strcopy(@h.res4,data.header[a].Sdata)
  else
    h.res4 := '';

//    videoplg : array[0..63] of char;
  a := getheader(data,'Video plugin');
  if a >= 0 then
    strcopy(@h.videoplg,data.header[a].Sdata)
  else
    h.videoplg := '';

//    soundplg : array[0..63] of char;
  a := getheader(data,'Sound Plugin');
  if a >= 0 then
    strcopy(@h.soundplg,data.header[a].Sdata)
  else
    h.soundplg := '';

//    inputplg : array[0..63] of char;
  a := getheader(data,'Input Plugin');
  if a >= 0 then
    strcopy(@h.inputplg,data.header[a].Sdata)
  else
    h.inputplg := '';

//    rspplg   : array[0..63] of char;
  a := getheader(data,'RSP Plugin');
  if a >= 0 then
    strcopy(@h.rspplg,data.header[a].Sdata)
  else
    h.rspplg := '';

//    author   : array[0..221] of char;
  a := getheader(data,'Author');
  if a >= 0 then
    strcopy(@h.author,data.header[a].Sdata)
  else
    h.author := '';

//    desc     : array[0..255] of char;
  a := getheader(data,'Description');
  if a >= 0 then
    strcopy(@h.desc,data.header[a].Sdata)
  else
    h.desc := '';

//    start    : longword;
  blockwrite(f,h,sizeof(h));
  //movie
  setlength(cdata,data.frames+1);
  for a := 0 to data.frames do
    cdata[a] := data.controller[0].data[a];
  blockwrite(f,cdata[0],(data.frames+1)*4);
  close(f);
end;

Procedure GetCodes (s : PChar);
begin
  strcopy(s,chars);
end;

Procedure CheckHeader(const fn : pchar; var res : boolean);
var
  F : file of char;
  fnn :string;
  h : header;
begin
res := false;
fnn := fn;
assign(f,fnn); reset(f);
if filesize(f) < sizeof(h) then
begin
  close(f); exit;
end;
blockread(f,h,sizeof(h));
if h.m64 <> 'M64'#$1a then
begin
  close(f); exit;
end;
if filesize(f) < $400 -4 + h.inputs*4 then
begin
  close(f); exit;
end;
res := true;
end;

exports GetInfo, LoadMovie, GetCodes, SaveMovie, CheckHeader;

end.
