unit tasmovies;

interface

uses sysutils;

type
  TMovie = record
    filename : PChar; // original filename
    headers : longword;
    header : array of record
      name : PChar; // name of header entry
      tp : byte; // type: 0 = signed, 1 = unsigned, 2 = flags, 3 = string
      l : integer; // length of data
      Sdata : pchar; // if value is string
      Idata : longword; // if value is numeric
    end;
    frames : longword; // movie length
    datastart : longword; // data start offset in original file
    controllers : byte; // number of controllers
    controller : array of record // up to 4 controllers
      data : array of longword; // up to 32 bits for every frame.
    end;
    fragments: longword; // number of unconverted fragments
    fragment : array of record
      name : pchar; // name for fragment (conversion purposes)
      size : longword; // size of fragment
      start : longword; // where it was originally in file
      data : array of byte;
    end;
    error : byte;
  end;


function getheader(const mov : TMovie; s : string) : integer;
function getfragment(const mov : TMovie; s : string) : integer;
function codetotext(const mov : TMovie; y : integer) : string;
function texttocode(s,gc : string) : integer;

implementation

function getheader;
var
  a : integer;
begin
  result := -1;
  for a := 0 to mov.headers-1 do
  begin
    if lowercase(mov.header[a].name) = lowercase(s) then result := a;
  end;
end;

function getfragment;
var a : integer;
begin
  result := -1;
  for a := 0 to mov.fragments-1 do
  begin
    if lowercase(mov.fragment[a].name) = lowercase(s) then result := a;
  end;
end;

function codetotext;
var
  a,d,e : integer;
  x,z,l : longword;
  s : string;
begin
  a := getheader(mov,'controller codes');
  if a = -1 then
    s := '0123456789abcdefghijklmonpqrstuv'
  else
    s := mov.header[a].Sdata + #0;
  result := '';
  x := 1;
  z := 0;
  e := 0;
  for d := 1 to length(s)-1 do
  begin
    inc(e);
    if s[d] = s[d+1] then
      x := (x shl 1) or x
    else
    begin
      l := shortint((y and x) shr z);
      if e = 1 then
      if l = 1 then result := result + s[d];
      if e > 1 then
        if l > 0 then
          result := result + s[d] + '=' + inttostr(shortint(l)) + ' ';
      x := 1 shl (d);
      z := d;
      e := 0;
    end;
  end;
end;

function texttocode;
var
  a,d : integer;
  l,x,y,t : longword;
  c : char;
begin
  s := s + ' ';
  t := 0;
  while length(s) > 1 do
  begin
    if pos(s[1],gc) = 0 then
    begin
      delete(s,1,1);
      continue
    end;
    l := 1;
    c := s[1];
    delete(s,1,1);
    if s[1] = '=' then
    begin
      l := strtoint(copy(s,2,pos(' ',s)-2));
      delete(s,1,pos(' ',s));
    end;
    x := 0;
    y := 0;
    for d := length(gc) downto 1do
    begin
      x := x shl 1;
      inc(y);
      if gc[d] = c then
      begin
        x := x or 1;
        y := 0;
      end;
    end;
    l := l shl y and x;
    t := t or l;
  end;
  result := t;
end;

end.
